#include<iostream>
#include<complex>
#define complex complex<double>

#include<bar.h>
#include<fluid.h>
#include<grid.h>
#include<hamop.h>
#include<wavefunction.h>
#include<../base/kbhit.cc> // kbhit() for unix 
                           // (cf. http://www.ohse.de/uwe/articles/kbhit.html )


// *** potentials, nuclear_charge, and laser pulse parameters
//     are declared in potentials.cc ***
#include "potentials.cc"

int main(int argc, char **argv)
{
  //
  // variables
  //
  char cstr_fname_logfi[1000], cstr_fname_wf_ini[1000], 
       cstr_fname_obser[1000], cstr_fname_wf_fin[1000],
       cstr_fname_info[1000];
  double imag_timestep, E_tot, E_tot_prev;
  int iinitmode, i, me, iv, ikey;
  long lno_of_ts, ts;   
  FILE *file_logfi, *file_wf_ini, *file_obser_imag, *file_wf_fin, *file_info;

  complex timestep;

  fluid ells;
  grid g;
  hamop hamilton;
  wavefunction staticpot, wf;

  fprintf(stdout," HYDROGEN_IM: Imaginary-time propagation for hydrogen atom\n");
  fprintf(stdout," (C) Copyright by Bauer D and Koval P, Heidelberg (2005)\n");
  fprintf(stdout," -------------------------------------------------------\n");

  //
  // input
  //
  // *** declare the grid ***
  g.set_dim(34); // 44 elliptical polariz., 34 linear polariz.
  g.set_ngps(2000,2,1);  // N_r, L, N
  g.set_delt(0.1/nuclear_charge);  // delta r
  g.set_offs(0,0,0);     // there is no offset in r, l, and N

  iinitmode = 2; // 1 -- random, 2 -- hydrogenic wf.

  iv        = 1; // verbosity of stdout

  sprintf(cstr_fname_logfi, "/home/theo/dbauer/res/argon/Ar_im-%d-%ld-%.2le.log",
    g.dimens(), g.ngps_x(), g.delt_x());
  sprintf(cstr_fname_wf_ini,"/home/theo/dbauer/res/argon/Ar_im_wf_ini-%d-%ld-%.2le.dat",
    g.dimens(), g.ngps_x(), g.delt_x());
  sprintf(cstr_fname_obser, "/home/theo/dbauer/res/argon/Ar_im_obser-%d-%ld-%.2le.dat",
    g.dimens(), g.ngps_x(), g.delt_x());
  sprintf(cstr_fname_wf_fin,"/home/theo/dbauer/res/argon/Ar_im_wf_fin-%d-%ld-%.2le.dat",
    g.dimens(), g.ngps_x(), g.delt_x());
  sprintf(cstr_fname_info,  "/home/theo/dbauer/res/argon/Ar_im_info-%d-%ld-%.2le.dat",
    g.dimens(), g.ngps_x(), g.delt_x());

  //
  // *** prepare for propagation ... ***
  // 

  // *** number of imaginary time steps ***
  lno_of_ts = 640000;      // upper limit of the number of time steps

  ells.init(g.ngps_z());

  ells[0]             = 1; // populated l quantum number (p state)

  me                  = 0; // dummy here


  // *** declare rest of variables ***
  imag_timestep=g.delt_x()/4.0;

  // the Hamiltonian
  hamilton.init(g,vecpot_x,vecpot_y,vecpot_z,scalarpotx,scalarpoty,scalarpotz,
                  imagpot,field);

  // this is the linear and constant part of the Hamiltonian
  staticpot.init(g.size()); 
  staticpot.calculate_staticpot(g,hamilton);

  // *** create some files with appropriate appendices
  file_logfi      = fopen(cstr_fname_logfi,"w");
  if (file_logfi==NULL)
  {fprintf(stderr, "%s cant be opened!\n", cstr_fname_logfi); exit(-2);}

  file_obser_imag = fopen(cstr_fname_obser,"w");
  if (file_obser_imag==NULL)
  {fprintf(stderr, "%s cant be opened!\n", cstr_fname_obser); exit(-3);}

  file_wf_ini    = fopen(cstr_fname_wf_ini,"w");
  if (file_wf_ini==NULL)
  {fprintf(stderr, "%s cant be opened!\n", cstr_fname_wf_ini); exit(-4);}

  if (iv!=0)
  {
     fprintf(stderr, "%s will be (re)written.\n", cstr_fname_logfi);
     fprintf(stderr, "%s will be (re)written.\n", cstr_fname_wf_ini);
     fprintf(stderr, "%s will be (re)written.\n", cstr_fname_obser);
     fprintf(stderr, "%s will be (re)written.\n", cstr_fname_wf_fin);
     fprintf(stderr, "%s will be (re)written.\n", cstr_fname_info);
  }

  ts=0;
  
  // *** wavefunction initialization ***
  wf.init(g.size()); 
  wf.init(g,iinitmode,1.0,ells);
  wf.normalize(g);

  fprintf(stdout,"Norm: %le\n",wf.norm(g));
  
  //
  // write in log file
  //
  fprintf(file_logfi,"Imaginary-time propagation\n");
  fprintf(file_logfi,"Grid: \n", g.ngps_x());
  fprintf(file_logfi,"g.ngps_x() = %ld\n", g.ngps_x());
  fprintf(file_logfi,"g.ngps_y() = %ld\n", g.ngps_y());
  fprintf(file_logfi,"g.ngps_z() = %ld\n", g.ngps_z());
  fprintf(file_logfi,"g.dimens() = %ld\n\n", g.dimens());
  fprintf(file_logfi,"g.delt_x() = %20.15le\n", g.delt_x());

  fprintf(file_logfi,"imag_timestep     = %20.15le\n", imag_timestep);
  fprintf(file_logfi,"lno_of_ts         = %ld\n", lno_of_ts);
  fprintf(file_logfi,"nuclear_charge    = %20.15le\n", nuclear_charge);
  fprintf(file_logfi,"iinitmode         = %d\n", iinitmode);
  fprintf(file_logfi,"cstr_fname_wf_ini = %s\n", cstr_fname_wf_ini);
  fprintf(file_logfi,"cstr_fname_obser  = %s\n", cstr_fname_obser);
  fprintf(file_logfi,"cstr_fname_wf_fin = %s\n", cstr_fname_wf_fin);
  fprintf(file_logfi,"cstr_fname_info   = %s\n", cstr_fname_info);
  fflush(file_logfi);


  // *** the orbitals before relaxation
  wf.dump_to_file_sh(g,file_wf_ini,1);
  fclose(file_wf_ini);

  // ********************************************************
  // ***** imaginary propagation ****************************
  // ********************************************************


  E_tot = 0.0;
  timestep=complex(0.0,-1.0*imag_timestep);  

  printf("Press any key to finish imaginary-time propagation...\n");
  if ((iv==1) && (is_time(0.1)))
fprintf(stdout,
" timestep        E_tot\n");

  // *** imaginary time propagation ***
  for (ts=0; ts<lno_of_ts; ts++)
  {
    //
    // calculate the total energy
    // 
    E_tot_prev = E_tot;
    E_tot = real(wf.energy(0.0,g,hamilton,me,staticpot,nuclear_charge));
    fprintf(file_obser_imag,"%6ld   % 20.15le\n",
      ts,E_tot);

    if ((iv==1) && (is_time(0.1)))
      fprintf(stdout,"%6ld        %20.15le\n",ts,E_tot);

    wf.propagate(timestep, 0.0, g, hamilton, me, staticpot, 0, nuclear_charge);

    wf.normalize(g);

    //
    // exclude the following and adjust lno_of_ts 
    // if you have problems with kbhit()
    //
    ikey=kbhit(0);
    if (ikey==-1) {fprintf(stderr, "err: kbhit(0)==-1\n"); exit(1);}
    if (ikey) break;
  }
 


  fclose(file_obser_imag);
  fclose(file_logfi);

  // *** store after imaginary time propagation
  file_wf_fin    = fopen(cstr_fname_wf_fin,"w");
  if (file_wf_fin==NULL)
  {fprintf(stderr, "%s cant be opened!\n", cstr_fname_wf_fin); exit(-5);}
  wf.dump_to_file_sh(g,file_wf_fin,1);
  fclose(file_wf_fin);

  file_info = fopen(cstr_fname_info, "w");
  fprintf(file_info,"<nuclear_charge>%20.15le</nuclear_charge>\n",
    nuclear_charge);
  fprintf(file_info,"<ngps_x>%ld</ngps_x>\n",g.ngps_x());
  fprintf(file_info,"<ngps_y>%ld</ngps_y>\n",g.ngps_y());
  fprintf(file_info,"<ngps_z>%ld</ngps_z>\n",g.ngps_z());
  fprintf(file_info,"<delt_x>%20.15le</delt_x>\n",g.delt_x());
  fprintf(file_info,"<dimens>%ld</dimens>\n",g.dimens());
  fclose(file_info);

  if (iv!=0)
  {
     fprintf(stderr, "%s is written.\n", cstr_fname_logfi);
     fprintf(stderr, "%s is written.\n", cstr_fname_wf_ini);
     fprintf(stderr, "%s is written.\n", cstr_fname_obser);
     fprintf(stderr, "%s is written.\n", cstr_fname_wf_fin);
     fprintf(stderr, "%s is written.\n", cstr_fname_info);
  }

  fprintf(stdout, "Hasta la vista...\n");
}

// 
// end of main program
// 
