double nuclear_charge = 1.0;  // hydrogen
double n_c=20.0;  // trapezoidal pulse: no. of cycles of constant intensity
                  // sinusoidal pulse: total no. of cycles
double n_r=5.0;   // trapezoidal pulse: up and down ramp over n_r cycles
double phi=M_PI / 2.0;  // absolute phase
double w=0.17;     // frequ. in au
double E_0=0.0;   // electric field amplitude 
                  // (will be assigned in hydrogen_re.cc)
double duration;  // will be assigned when vecpot_z is called
double ampl_im=100.0;  // amplitude of imaginary absorbing potential

//
// functions
//

double vecpot_sin2_shape(double time)
{
  double result=0.0;
  double ww;

  ww=w/(2.0*n_c);
  duration=n_c*2*M_PI/w;
 
  if ((time>0.0) && (time<duration))
  {
    result=E_0/(4.0*(w*w*w-4.0*w*ww*ww))*(-8.0*ww*ww*sin(phi)-
    2*(w*w-4*ww*ww)*sin(w*time+phi)+w*((w+2*ww)*
    sin(w*time-2*ww*time+phi)+(w-2*ww)*sin(w*time+2*ww*time+phi)));
  }

  return result;
}                                                                                   


double vecpot_ramping(double time)
{
  double result, ampl, ramping, constperiod, downramp, T;
  result = 0.0;
  ampl   = E_0 / w;

  T = 2 * M_PI / w;

  constperiod = n_c * T;
  ramping     = n_r * T;
  downramp    = ramping;
  
  duration = ramping + downramp + constperiod;


  if (time>0.0)
    {


      if (time<ramping){
	result=-ampl/ramping*time*cos(w*time) + ampl/(w*ramping)*sin(w*time);}
      
      if ( (time>=ramping)&&(time<ramping+constperiod))
	result=-ampl*cos(w*time);
      
      if ( (time >= ramping+constperiod) && (time < ramping+constperiod+downramp)) 
	{  result=ampl/downramp*(time-downramp-constperiod-ramping)*
	     cos(w*time)-ampl/(w*downramp)*sin(w*time);
	}
      
      if ( time > ramping+constperiod+downramp ) result=0.0;
    };
  
  return result;
}  


//
double vecpot_x(double time, int me)
{
  return 0.0;
}  

//
double vecpot_y(double time, int me)
{
   return 0.0;
}  


double vecpot_z(double time, int me)
{
  //  return 0.0;
  //  return vecpot_ramping(time);  // choose trapezoidal
  return vecpot_sin2_shape(time);  // or sin^2-shaped pulse
}


double scalarpotx(double x, double y, double z, double time, int me)
{  

  double result=-nuclear_charge/x;
  return result;
}



double scalarpoty(double x, double y, double z, double time, int me)
{
  double result=0.0;  
  return result;
}


double scalarpotz(double x, double y, double z, double time, int me)
{
  double result=0.0;
  return result;
}
  

double field(double time, int me)
{
  double result=0.0;
  return result;
}  


double imagpot(long xindex, long yindex, long zindex, double time, grid g)
{
  double x,y,z;

  if (ampl_im>1.0)
    {
      x=((double) xindex + 0.5  - 0.5*g.ngps_x())/(0.5*g.ngps_x())
	*((double) xindex + 0.5  - 0.5*g.ngps_x())/(0.5*g.ngps_x());
      
      switch(g.dimens())
	{
	case 3 : 
	  y=((double) yindex + 0.5  - 0.5*g.ngps_y())/(0.5*g.ngps_y())
	    *((double) yindex + 0.5  - 0.5*g.ngps_y())/(0.5*g.ngps_y());
	  z=((double) zindex + 0.5  - 0.5*g.ngps_z())/(0.5*g.ngps_z())
	    *((double) zindex + 0.5  - 0.5*g.ngps_z())/(0.5*g.ngps_z());
	  return ampl_im*(x*x*x*x*x*x*x*x+y*y*y*y*y*y*y*y+z*z*z*z*z*z*z*z);
	  break;
	  
	case 2 : case 6 :
	  y=((double) yindex + 0.5  - 0.5*g.ngps_y())/(0.5*g.ngps_y())
	    *((double) yindex + 0.5  - 0.5*g.ngps_y())/(0.5*g.ngps_y());
	  return ampl_im*(x*x*x*x*x*x*x*x+y*y*y*y*y*y*y*y);
	  break;
	  
	case 1 : case 5 : case 15 :
	  return ampl_im*(x*x*x*x*x*x*x*x);
	  break;
	  
	case 4 : case 14 : case 24 : case 34 :		
	  x=((double) xindex + 0.5  )/(g.ngps_x())
	    *((double) xindex + 0.5  )/(g.ngps_x());
	  return ampl_im*(x*x*x*x*x*x*x*x);
	  break;
	  
	default : return 0.0;
	}
    }
  else
    {
      return 0.0;
    };
      
}  


void project_output(FILE* os, wavefunction &wf)
{
  long i;
  for (i=0; i<wf.wf_size(); i++)
    {
      fprintf(os,"%e ",real(wf[i]*conj(wf[i])));
    };

}


// overloaded below
void observables_output(grid g, FILE* os, 
			const wavefunction &orb_energs, 
			const wavefunction &orb_hartrees, 
			const fluid &orb_norms)
{
  long no;

  for (no=0; no<g.ngps_z(); no++)
    {
      fprintf(os,"%16.12e %16.12e ",real(orb_energs[no])+ real(orb_hartrees[no]),orb_norms[no]);
    };

}

// overloaded above
void observables_output(grid g, FILE* os, 
			const wavefunction &orb_energs, 
			const wavefunction &orb_hartrees,
			const wavefunction &orb_fields,
			const fluid &orb_norms)
{
  long no;

  for (no=0; no<g.ngps_z(); no++)
    {
      fprintf(os,"%16.12e %16.12e ",real(orb_energs[no])+ real(orb_hartrees[no])+ real(orb_fields[no]),orb_norms[no]);
    };

}
